/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.client;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.pua.prot.manager.ProtocolRelatedOperation;
import de.bsvrz.pua.prot.util.JobInProgress;
import de.bsvrz.pua.prot.util.JobState;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.datk.PidProtocol;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;

/**
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
public class ProtocolCompletionAnswerReceiver extends ProtocolAnswerReceiver{

	private static final Debug _debug = Debug.getLogger();

	private final PuaClient _client;

	public ProtocolCompletionAnswerReceiver(ClientDavInterface dav, ClientApplication client, final long requestId, PuaClient puaClient){
		super(dav, client, requestId, false, 0);
		if(puaClient == null) throw new IllegalArgumentException("puaClient ist null");
		_client = puaClient;
		incoming = null;
	}

	@Override
	protected void update(final ResultData result) {
		if(result != null && result.hasData()){
			final Data data = result.getData();
			if(data.getTextValue("Operationscode").getText().equals("ProtokollFertigmeldung")) {
				final long id = data.getUnscaledValue(PidProtocol.protocolId).longValue();
				byte[] bytes = data.getUnscaledArray(PidProtocol.data).getByteArray();
				if(bytes.length == 0) {
					// Alte PuA-Server-Version sendet immer nur Protokoll-Fertigmeldungen
					_client.protocolFinished(createDummyJob(id));
				}
				else {
					byte signal = bytes[0];
					if(signal == 0){
						// Protokoll gelöscht
						_client.protocolRemoved(id);
					}
					else if(signal == 1){
						// Protokoll fertig
						ByteArrayInputStream in = new ByteArrayInputStream(bytes);
						in.skip(1); // erstes Byte überspringen
						try {
							ObjectInputStream inputStream = new ObjectInputStream(in);
							Object o = inputStream.readObject();
							inputStream.close();
							if(o instanceof JobInProgress) {
								JobInProgress jobInProgress = (JobInProgress) o;
								jobInProgress.initialize(dav.getDataModel());
								_client.protocolFinished(jobInProgress);
							}
						}
						catch(Exception e){
							_debug.warning("Kann Protokoll-Fertigmeldung nicht deserialisieren", e);
						}
					}
					else if(signal == 2){
						// Protokoll gelesen
						_client.protocolRead(id);
					}
					else if(signal == 3){
						_client.updateArsStatus(id != 0);
						if(bytes.length >= 2){
							// Ab Protokollversion 4 schickt der Server hiermit die aktuelle Server-Protokollversion
							_client.incrementProtocolVersion(bytes[1] & 0xFF);
						}
						else {
							// Ab Protokollversion 3 informiert der Server über den Archivzustand mit der ID 3
							_client.incrementProtocolVersion(3);
						}
					}
				}
			}
		}
	}

	private static JobInProgress createDummyJob(final long id) {
		try {
			_debug.warning("Es konnten nicht alle Details des Auftrags gelesen werden, da die Software des PuA-Servers veraltet ist.");
			return new JobInProgress(id, ProtocolRelatedOperation.CREATE, JobState.Finished, (byte)100, -1, -1, null);
		}
		catch(FailureException e) {
			// Sollte nicht auftreten
			throw new IllegalStateException(e);
		}
	}
}
