/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.client.dataobject;

import de.bsvrz.pua.prot.util.attributes.RealAttributeDescription;

/**
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class RealAttributeColumn extends Column {

	private static final long serialVersionUID = 6458597991768970034L;

	private final long _objectId;

	private final long _pseudoObjectIndex;

	private final String _alias;

	private final long[] _attributeGroupIds;

	private final long[] _aspectIds;

	private final String[] _names;

	private final short _simVar;

	/** @param rad Attributbeschreibung, auf der die Spalte basiert. */
	public RealAttributeColumn(RealAttributeDescription rad) {
		//XXX rad.attibute ist null wenn ein Alias verwendet wird!! GGf anders lösen
		super(ColumnType.REAL_ATTRIBUTE, rad.getPrettyName(), (rad.getAttribute() != null) ? rad.getAttribute().getId() : -1, rad.getColumnName());

		_objectId = (rad.qualifier.object != null) ? rad.qualifier.object.getId() : -1;
		_simVar = rad.qualifier.simVar;

		//referenztiefe ermitteln:
		int i = 1;
		RealAttributeDescription reference = rad.getReference();
		while(reference != null) {
			reference = reference.getReference();
			i++;
		}
		_attributeGroupIds = new long[i];
		_aspectIds = new long[i];
		_names = new String[i];
		for(i = 0, reference = rad; reference != null; i++, reference = reference.getReference()) {
			_attributeGroupIds[i] = reference.qualifier.attributeGroup.getId();
			_aspectIds[i] = reference.qualifier.aspect.getId();
			_names[i] = reference.getName();
		}

		_pseudoObjectIndex = rad.getIndex();

		_alias = rad.getAlias();
	}

	/** @return Returns the aspectIds. */
	public long[] getAspectIds() {
		return _aspectIds;
	}

	/** @return Returns the attributeGroupIds. */
	public long[] getAttributeGroupIds() {
		return _attributeGroupIds;
	}

	/** @return Returns the names. */
	public String[] getNames() {
		return _names;
	}

	/** @return Gibt die ID des Objektes dieser Spalte zurück. -1 falls es kein Objekt gibt (-&gt;Platzhalterspalte für Pseudoobjekte) */
	public long getObjectId() {
		return _objectId;
	}

	/** @return Returns the simVar. */
	public short getSimVar() {
		return _simVar;
	}

	/**
	 * Gibt den Index des Pseudoobjekt-Tupels zurück. Wird beispielsweise ein Skript mit folgenden Parametern gestartet: (fs, mq) = (fs1, mq1)(fs2, mq2) wird hier
	 * für fs1 und mq1 0 zurückgegeben und für fs2 und mq2 1.
	 *
	 * @return Pseudoobjekt-Index oder -1 falls kein Pseudoobjekt verwendet wurde.
	 */
	public long getPseudoObjectIndex() {
		return _pseudoObjectIndex;
	}

	/**
	 * Gibt den in dieser Spalte verwendeten Alias zurück. Wird beispielsweise ein Skript mit folgenden Parametern gestartet: (fs, mq) = (fs1, mq1)(fs2, mq2) wird hier
	 * für fs1 und fs2 "fs" zurückgegeben und für mq1 und mq2 "mq".
	 * @return Alias-Namen oder <code>null</code> falls kein Alias verwendet wurde.
	 */
	public String getAlias() {
		return _alias;
	}
}
