/*
 * Copyright 2016 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.manager.clientDatabase;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.sender.PuaSender;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * TBD Dokumentation
 *
 * @author Kappich Systemberatung
 */
public class ClientDatabase {
	
	private static final Map<SystemObject, Client> _clients = new HashMap<>();
	
	/** Unterstütze Protokoll-Version des Servers */
	public static final int CURRENT_PROTOCOL_VERSION = 4;
	
	private static final Debug _debug = Debug.getLogger();
	
	public static synchronized void addClient(SystemObject object, PuaSender sender, int protocolVersion){
		_debug.info("Neuer Client " + object + " mit Protokollversion " + protocolVersion + " hat sich angemeldet");
		_clients.put(object, new Client(object, sender, protocolVersion));
	}	
	
	public static synchronized PuaSender getSender(SystemObject object){
		Client client = _clients.get(object);
		if(client == null) return null;
		return client.getClientSender();
	}	
	
	public static synchronized int getProtocolVersion(SystemObject object){
		Client client = _clients.get(object);
		if(client == null) {
			// Client ist noch unbekannt, niedrigste Protokollversion annehmen.
			// Alte Clients melden sich nicht an den Listener an und verhandeln daher keine Protokollversion
			return 1;
		}
		return client.getProtocolVersion();
	}

	public static synchronized void removeClient(final SystemObject object) {
		if(_clients.remove(object) != null) {
			_debug.info("Client " + object + "hat sich abgemeldet");
		}
	}
	
	public static synchronized Map<SystemObject, Client> getClients(){
		return new HashMap<>(_clients);
	}

	public static class Client{
		
		private final SystemObject _object;
		
		private final PuaSender _clientSender;
		
		private final int _protocolVersion;

		public Client(final SystemObject object, final PuaSender clientSender, final int protocolVersion) {
			_object = object;
			_clientSender = clientSender;
			_protocolVersion = protocolVersion;
		}

		public PuaSender getClientSender() {
			return _clientSender;
		}

		public SystemObject getObject() {
			return _object;
		}

		public int getProtocolVersion() {
			return _protocolVersion;
		}
	}
}
