/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib;

import java.io.Serializable;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.puk.param.lib.ParameterInfo.SerializableParameterInfo;
import de.bsvrz.puk.param.lib.daten.DataWithTime;
import de.bsvrz.puk.param.lib.daten.Datensatz;
import de.bsvrz.puk.param.lib.daten.ParameterDatenFactory;

/**
 * Serialisierbare Form eines Parameters für die Übertragung als Bytearray
 * innerhalb einer Datenverteiler-Attributgruppe.
 *
 * @author BitCtrl Systems GmbH, Peuker
 * @version $Id$
 */
public final class SerializableParameter implements Serializable {

	/** Versions-ID zur Serialisierung. */
	private static final long serialVersionUID = 1L;

	/** die Parameteterspezifikation. */
	private final SerializableParameterInfo info;

	/** die Quelle für den Parameter. */
	private final SerializableQuelle quelle;

	/** der Zeitpunkt der Daten in Millisekunden. */
	private final long zeitpunktInMs;

	/** die Daten des Parameters. */
	private final Datensatz daten;

	/**
	 * erzeugt eine Instanz der Klasse auf Basis des entsprechenden
	 * Parameterobjekts.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param parameter
	 *            der zu Grunde liegende Parameter
	 */
	public SerializableParameter(final ClientDavInterface verbindung, final Parameter parameter) {
		info = parameter.getInfo().getSerializableParameterInfo(verbindung);
		if (null != parameter.getDataWithTime()) {
			daten = (Datensatz) ParameterDatenFactory.getSerializable(verbindung, parameter.getData());
			zeitpunktInMs = parameter.getDataWithTime().getZeitpunkt();
		} else {
			daten = null;
			zeitpunktInMs = 0;
		}
		if (null != parameter.getQuelle()) {
			quelle = parameter.getQuelle().getSerializable();
		} else {
			quelle = null;
		}
	}

	/**
	 * erzeugt die dem Objekt entsprechende {@link Parameter}-Instanz, in der
	 * die entsprechenden Datenobjekte als DAF-Objekte repräsentiert sind.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @return das {@link Parameter}-Objekt
	 */
	public Parameter erzeugeParameter(final ClientDavInterface verbindung) {
		final DataModel model = verbindung.getDataModel();
		final Parameter parameter = new Parameter(info.erzeugeParameterInfo(verbindung));
		if (null != quelle) {
			parameter.setQuelle(model.getObject(quelle.getObjektId()), quelle.getTyp());
			parameter.getQuelle().setDataAvailable(quelle.isDataAvailable());
		}
		if (null != daten) {
			parameter.setDataWithTime(new DataWithTime(
					ParameterDatenFactory.getData(verbindung, parameter.getInfo().getAtg(), daten), zeitpunktInMs));
		}
		return parameter;
	}
}
